@@ -4,7 +4,7 @@ import urwid
 
 
 class Pudding(urwid.Widget):
-    _sizing = frozenset(['flow'])
+    _sizing = frozenset(["flow"])
 
     def rows(self, size, focus=False):
         return 1
@@ -16,10 +16,9 @@ class Pudding(urwid.Widget):
 
 
 class BoxPudding(urwid.Widget):
-    _sizing = frozenset(['box'])
+    _sizing = frozenset(["box"])
 
     def render(self, size, focus=False):
         (maxcol, maxrow) = size
         num_pudding = maxcol / len("Pudding")
-        return urwid.TextCanvas(["Pudding" * num_pudding] * maxrow,
-                                maxcol=maxcol)
+        return urwid.TextCanvas(["Pudding" * num_pudding] * maxrow, maxcol=maxcol)
