@@ -4,10 +4,11 @@ import urwid
 
 
 class SelectablePudding(urwid.Widget):
-    _sizing = frozenset(['flow'])
+    _sizing = frozenset(["flow"])
     _selectable = True
 
     def __init__(self):
+        super().__init__()
         self.pudding = "pudding"
 
     def rows(self, size, focus=False):
@@ -19,8 +20,7 @@ class SelectablePudding(urwid.Widget):
         pudding = self.pudding
         if focus:
             pudding = pudding.upper()
-        return urwid.TextCanvas([pudding * num_pudding],
-            maxcol=maxcol)
+        return urwid.TextCanvas([pudding * num_pudding], maxcol=maxcol)
 
     def keypress(self, size, key):
         (maxcol,) = size
@@ -29,7 +29,7 @@ class SelectablePudding(urwid.Widget):
         if key.lower() in self.pudding:
             # remove letter from pudding
             n = self.pudding.index(key.lower())
-            self.pudding = self.pudding[:n] + self.pudding[n+1:]
+            self.pudding = self.pudding[:n] + self.pudding[n + 1 :]
             if not self.pudding:
                 self.pudding = "pudding"
             self._invalidate()
