@@ -4,15 +4,16 @@ import urwid
 
 
 def exit_on_q(key):
-    if key in ('q', 'Q'):
+    if key in ("q", "Q"):
         raise urwid.ExitMainLoop()
 
+
 class QuestionBox(urwid.Filler):
     def keypress(self, size, key):
-        if key != 'enter':
+        if key != "enter":
             return super().keypress(size, key)
-        self.original_widget = urwid.Text(
-            f"Nice to meet you,\n{edit.edit_text}.\n\nPress Q to exit.")
+        self.original_widget = urwid.Text(f"Nice to meet you,\n{edit.edit_text}.\n\nPress Q to exit.")
+
 
 edit = urwid.Edit("What is your name?\n")
 fill = QuestionBox(edit)
