@@ -19,7 +19,7 @@
 #    License along with this library; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# Urwid web site: http://excess.org/urwid/
+# Urwid web site: https://urwid.org/
 
 """
 Urwid example lazy directory browser / tree view
@@ -31,40 +31,43 @@ Features:
 - outputs a quoted list of files and directories "selected" on exit
 """
 
-from __future__ import print_function
+
+from __future__ import annotations
 
 import itertools
-import re
 import os
+import re
+import typing
 
 import urwid
 
+if typing.TYPE_CHECKING:
+    from collections.abc import Hashable
+
 
 class FlagFileWidget(urwid.TreeWidget):
     # apply an attribute to the expand/unexpand icons
-    unexpanded_icon = urwid.AttrMap(urwid.TreeWidget.unexpanded_icon,
-        'dirmark')
-    expanded_icon = urwid.AttrMap(urwid.TreeWidget.expanded_icon,
-        'dirmark')
+    unexpanded_icon = urwid.AttrMap(urwid.TreeWidget.unexpanded_icon, "dirmark")
+    expanded_icon = urwid.AttrMap(urwid.TreeWidget.expanded_icon, "dirmark")
 
-    def __init__(self, node):
-        self.__super.__init__(node)
+    def __init__(self, node: urwid.TreeNode) -> None:
+        super().__init__(node)
         # insert an extra AttrWrap for our own use
-        self._w = urwid.AttrWrap(self._w, None)
+        self._w = urwid.AttrMap(self._w, None)
         self.flagged = False
         self.update_w()
 
-    def selectable(self):
+    def selectable(self) -> bool:
         return True
 
-    def keypress(self, size, key):
+    def keypress(self, size, key: str) -> str | None:
         """allow subclasses to intercept keystrokes"""
-        key = self.__super.keypress(size, key)
+        key = super().keypress(size, key)
         if key:
             key = self.unhandled_keys(size, key)
         return key
 
-    def unhandled_keys(self, size, key):
+    def unhandled_keys(self, size, key: str) -> str | None:
         """
         Override this method to intercept keystrokes in subclasses.
         Default behavior: Toggle flagged on space, ignore other keys.
@@ -72,105 +75,106 @@ class FlagFileWidget(urwid.TreeWidget):
         if key == " ":
             self.flagged = not self.flagged
             self.update_w()
-        else:
-            return key
+            return None
 
-    def update_w(self):
-        """Update the attributes of self.widget based on self.flagged.
-        """
+        return key
+
+    def update_w(self) -> None:
+        """Update the attributes of self.widget based on self.flagged."""
         if self.flagged:
-            self._w.attr = 'flagged'
-            self._w.focus_attr = 'flagged focus'
+            self._w.attr = "flagged"
+            self._w.focus_attr = "flagged focus"
         else:
-            self._w.attr = 'body'
-            self._w.focus_attr = 'focus'
+            self._w.attr = "body"
+            self._w.focus_attr = "focus"
 
 
 class FileTreeWidget(FlagFileWidget):
     """Widget for individual files."""
-    def __init__(self, node):
-        self.__super.__init__(node)
+
+    def __init__(self, node: FileNode) -> None:
+        super().__init__(node)
         path = node.get_value()
         add_widget(path, self)
 
-    def get_display_text(self):
+    def get_display_text(self) -> str | tuple[Hashable, str] | list[str | tuple[Hashable, str]]:
         return self.get_node().get_key()
 
 
-
 class EmptyWidget(urwid.TreeWidget):
     """A marker for expanded directories with no contents."""
-    def get_display_text(self):
-        return ('flag', '(empty directory)')
+
+    def get_display_text(self) -> str | tuple[Hashable, str] | list[str | tuple[Hashable, str]]:
+        return ("flag", "(empty directory)")
 
 
 class ErrorWidget(urwid.TreeWidget):
     """A marker for errors reading directories."""
 
-    def get_display_text(self):
-        return ('error', "(error/permission denied)")
+    def get_display_text(self) -> str | tuple[Hashable, str] | list[str | tuple[Hashable, str]]:
+        return ("error", "(error/permission denied)")
 
 
 class DirectoryWidget(FlagFileWidget):
     """Widget for a directory."""
-    def __init__(self, node):
-        self.__super.__init__(node)
+
+    def __init__(self, node: DirectoryNode) -> None:
+        super().__init__(node)
         path = node.get_value()
         add_widget(path, self)
         self.expanded = starts_expanded(path)
         self.update_expanded_icon()
 
-    def get_display_text(self):
+    def get_display_text(self) -> str | tuple[Hashable, str] | list[str | tuple[Hashable, str]]:
         node = self.get_node()
         if node.get_depth() == 0:
             return "/"
-        else:
-            return node.get_key()
+
+        return node.get_key()
 
 
 class FileNode(urwid.TreeNode):
     """Metadata storage for individual files"""
 
-    def __init__(self, path, parent=None):
+    def __init__(self, path: str, parent: urwid.ParentNode | None = None) -> None:
         depth = path.count(dir_sep())
         key = os.path.basename(path)
-        urwid.TreeNode.__init__(self, path, key=key, parent=parent, depth=depth)
+        super().__init__(path, key=key, parent=parent, depth=depth)
 
-    def load_parent(self):
-        parentname, myname = os.path.split(self.get_value())
+    def load_parent(self) -> DirectoryNode:
+        parentname, _myname = os.path.split(self.get_value())
         parent = DirectoryNode(parentname)
         parent.set_child_node(self.get_key(), self)
         return parent
 
-    def load_widget(self):
+    def load_widget(self) -> FileTreeWidget:
         return FileTreeWidget(self)
 
 
 class EmptyNode(urwid.TreeNode):
-    def load_widget(self):
+    def load_widget(self) -> EmptyWidget:
         return EmptyWidget(self)
 
 
 class ErrorNode(urwid.TreeNode):
-    def load_widget(self):
+    def load_widget(self) -> ErrorWidget:
         return ErrorWidget(self)
 
 
 class DirectoryNode(urwid.ParentNode):
     """Metadata storage for directories"""
 
-    def __init__(self, path, parent=None):
+    def __init__(self, path: str, parent: urwid.ParentNode | None = None) -> None:
         if path == dir_sep():
             depth = 0
             key = None
         else:
             depth = path.count(dir_sep())
             key = os.path.basename(path)
-        urwid.ParentNode.__init__(self, path, key=key, parent=parent,
-                                  depth=depth)
+        super().__init__(path, key=key, parent=parent, depth=depth)
 
-    def load_parent(self):
-        parentname, myname = os.path.split(self.get_value())
+    def load_parent(self) -> DirectoryNode:
+        parentname, _myname = os.path.split(self.get_value())
         parent = DirectoryNode(parentname)
         parent.set_child_node(self.get_key(), self)
         return parent
@@ -182,14 +186,13 @@ class DirectoryNode(urwid.ParentNode):
             path = self.get_value()
             # separate dirs and files
             for a in os.listdir(path):
-                if os.path.isdir(os.path.join(path,a)):
+                if os.path.isdir(os.path.join(path, a)):
                     dirs.append(a)
                 else:
                     files.append(a)
-        except OSError as e:
+        except OSError:
             depth = self.get_depth() + 1
-            self._children[None] = ErrorNode(self, parent=self, key=None,
-                                             depth=depth)
+            self._children[None] = ErrorNode(self, parent=self, key=None, depth=depth)
             return [None]
 
         # sort dirs and files
@@ -200,87 +203,93 @@ class DirectoryNode(urwid.ParentNode):
         # collect dirs and files together again
         keys = dirs + files
         if len(keys) == 0:
-            depth=self.get_depth() + 1
-            self._children[None] = EmptyNode(self, parent=self, key=None,
-                                             depth=depth)
+            depth = self.get_depth() + 1
+            self._children[None] = EmptyNode(self, parent=self, key=None, depth=depth)
             keys = [None]
         return keys
 
-    def load_child_node(self, key):
+    def load_child_node(self, key) -> EmptyNode | DirectoryNode | FileNode:
         """Return either a FileNode or DirectoryNode"""
         index = self.get_child_index(key)
         if key is None:
             return EmptyNode(None)
-        else:
-            path = os.path.join(self.get_value(), key)
-            if index < self.dir_count:
-                return DirectoryNode(path, parent=self)
-            else:
-                path = os.path.join(self.get_value(), key)
-                return FileNode(path, parent=self)
 
-    def load_widget(self):
+        path = os.path.join(self.get_value(), key)
+        if index < self.dir_count:
+            return DirectoryNode(path, parent=self)
+
+        path = os.path.join(self.get_value(), key)
+        return FileNode(path, parent=self)
+
+    def load_widget(self) -> DirectoryWidget:
         return DirectoryWidget(self)
 
 
 class DirectoryBrowser:
-    palette = [
-        ('body', 'black', 'light gray'),
-        ('flagged', 'black', 'dark green', ('bold','underline')),
-        ('focus', 'light gray', 'dark blue', 'standout'),
-        ('flagged focus', 'yellow', 'dark cyan',
-                ('bold','standout','underline')),
-        ('head', 'yellow', 'black', 'standout'),
-        ('foot', 'light gray', 'black'),
-        ('key', 'light cyan', 'black','underline'),
-        ('title', 'white', 'black', 'bold'),
-        ('dirmark', 'black', 'dark cyan', 'bold'),
-        ('flag', 'dark gray', 'light gray'),
-        ('error', 'dark red', 'light gray'),
-        ]
+    palette: typing.ClassVar[list[tuple[str, str, str, ...]]] = [
+        ("body", "black", "light gray"),
+        ("flagged", "black", "dark green", ("bold", "underline")),
+        ("focus", "light gray", "dark blue", "standout"),
+        ("flagged focus", "yellow", "dark cyan", ("bold", "standout", "underline")),
+        ("head", "yellow", "black", "standout"),
+        ("foot", "light gray", "black"),
+        ("key", "light cyan", "black", "underline"),
+        ("title", "white", "black", "bold"),
+        ("dirmark", "black", "dark cyan", "bold"),
+        ("flag", "dark gray", "light gray"),
+        ("error", "dark red", "light gray"),
+    ]
 
-    footer_text = [
-        ('title', "Directory Browser"), "    ",
-        ('key', "UP"), ",", ('key', "DOWN"), ",",
-        ('key', "PAGE UP"), ",", ('key', "PAGE DOWN"),
+    footer_text: typing.ClassVar[list[tuple[str, str] | str]] = [
+        ("title", "Directory Browser"),
+        "    ",
+        ("key", "UP"),
+        ",",
+        ("key", "DOWN"),
+        ",",
+        ("key", "PAGE UP"),
+        ",",
+        ("key", "PAGE DOWN"),
         "  ",
-        ('key', "SPACE"), "  ",
-        ('key', "+"), ",",
-        ('key', "-"), "  ",
-        ('key', "LEFT"), "  ",
-        ('key', "HOME"), "  ",
-        ('key', "END"), "  ",
-        ('key', "Q"),
-        ]
+        ("key", "SPACE"),
+        "  ",
+        ("key", "+"),
+        ",",
+        ("key", "-"),
+        "  ",
+        ("key", "LEFT"),
+        "  ",
+        ("key", "HOME"),
+        "  ",
+        ("key", "END"),
+        "  ",
+        ("key", "Q"),
+    ]
 
-
-    def __init__(self):
+    def __init__(self) -> None:
         cwd = os.getcwd()
         store_initial_cwd(cwd)
         self.header = urwid.Text("")
         self.listbox = urwid.TreeListBox(urwid.TreeWalker(DirectoryNode(cwd)))
         self.listbox.offset_rows = 1
-        self.footer = urwid.AttrWrap(urwid.Text(self.footer_text),
-            'foot')
+        self.footer = urwid.AttrMap(urwid.Text(self.footer_text), "foot")
         self.view = urwid.Frame(
-            urwid.AttrWrap(self.listbox, 'body'),
-            header=urwid.AttrWrap(self.header, 'head'),
-            footer=self.footer)
+            urwid.AttrMap(self.listbox, "body"), header=urwid.AttrMap(self.header, "head"), footer=self.footer
+        )
 
-    def main(self):
+    def main(self) -> None:
         """Run the program."""
 
-        self.loop = urwid.MainLoop(self.view, self.palette,
-            unhandled_input=self.unhandled_input)
+        self.loop = urwid.MainLoop(self.view, self.palette, unhandled_input=self.unhandled_input)
         self.loop.run()
 
         # on exit, write the flagged filenames to the console
         names = [escape_filename_sh(x) for x in get_flagged_names()]
         print(" ".join(names))
 
-    def unhandled_input(self, k):
+    def unhandled_input(self, k: str | tuple[str, int, int, int]) -> None:
         # update display of focus directory
-        if k in ('q','Q'):
+        if k in {"q", "Q"}:
             raise urwid.ExitMainLoop()
 
 
@@ -288,55 +297,53 @@ def main():
     DirectoryBrowser().main()
 
 
-
-
 #######
 # global cache of widgets
 _widget_cache = {}
 
+
 def add_widget(path, widget):
     """Add the widget for a given path"""
 
     _widget_cache[path] = widget
 
-def get_flagged_names():
+
+def get_flagged_names() -> list[str]:
     """Return a list of all filenames marked as flagged."""
 
-    l = []
-    for w in _widget_cache.values():
-        if w.flagged:
-            l.append(w.get_node().get_value())
-    return l
-
+    names = [w.get_node().get_value() for w in _widget_cache.values() if w.flagged]
+    return names
 
 
 ######
 # store path components of initial current working directory
 _initial_cwd = []
 
-def store_initial_cwd(name):
+
+def store_initial_cwd(name: str) -> None:
     """Store the initial current working directory path components."""
 
-    global _initial_cwd
-    _initial_cwd = name.split(dir_sep())
+    _initial_cwd.clear()
+    _initial_cwd.extend(name.split(dir_sep()))
 
-def starts_expanded(name):
+
+def starts_expanded(name: str) -> bool:
     """Return True if directory is a parent of initial cwd."""
 
-    if name is '/':
+    if name == "/":
         return True
 
-    l = name.split(dir_sep())
-    if len(l) > len(_initial_cwd):
+    path_elements = name.split(dir_sep())
+    if len(path_elements) > len(_initial_cwd):
         return False
 
-    if l != _initial_cwd[:len(l)]:
+    if path_elements != _initial_cwd[: len(path_elements)]:
         return False
 
     return True
 
 
-def escape_filename_sh(name):
+def escape_filename_sh(name: str) -> str:
     """Return a hopefully safe shell-escaped version of a filename."""
 
     # check whether we have unprintable characters
@@ -346,45 +353,44 @@ def escape_filename_sh(name):
             return escape_filename_sh_ansic(name)
 
     # all printable characters, so return a double-quoted version
-    name.replace('\\','\\\\')
-    name.replace('"','\\"')
-    name.replace('`','\\`')
-    name.replace('$','\\$')
-    return '"'+name+'"'
+    name = name.replace("\\", "\\\\").replace('"', '\\"').replace("`", "\\`").replace("$", "\\$")
+    return f'"{name}"'
 
 
-def escape_filename_sh_ansic(name):
+def escape_filename_sh_ansic(name: str) -> str:
     """Return an ansi-c shell-escaped version of a filename."""
 
-    out =[]
+    out = []
     # gather the escaped characters into a list
     for ch in name:
         if ord(ch) < 32:
-            out.append("\\x%02x"% ord(ch))
-        elif ch == '\\':
-            out.append('\\\\')
+            out.append(f"\\x{ord(ch):02x}")
+        elif ch == "\\":
+            out.append("\\\\")
         else:
             out.append(ch)
 
     # slap them back together in an ansi-c quote  $'...'
-    return "$'" + "".join(out) + "'"
+    return f"$'{''.join(out)}'"
+
+
+SPLIT_RE = re.compile(r"[a-zA-Z]+|\d+")
+
 
-SPLIT_RE = re.compile(r'[a-zA-Z]+|\d+')
 def alphabetize(s):
     L = []
     for isdigit, group in itertools.groupby(SPLIT_RE.findall(s), key=lambda x: x.isdigit()):
         if isdigit:
-            for n in group:
-                L.append(('', int(n)))
+            L.extend(("", int(n)) for n in group)
         else:
-            L.append((''.join(group).lower(), 0))
+            L.append(("".join(group).lower(), 0))
     return L
 
-def dir_sep():
+
+def dir_sep() -> str:
     """Return the separator used in this os."""
-    return getattr(os.path,'sep','/')
+    return getattr(os.path, "sep", "/")
 
 
-if __name__=="__main__":
+if __name__ == "__main__":
     main()
-
