@@ -21,12 +21,13 @@
 
 from __future__ import annotations
 
+import typing
 from contextlib import suppress
 
 import urwid
 
 
-def main():
+def main() -> None:
     urwid.set_encoding("utf8")
     term = urwid.Terminal(None, encoding="utf-8")
 
@@ -35,24 +36,24 @@ def main():
     mainframe = urwid.LineBox(
         urwid.Pile(
             [
-                ("weight", 70, term),
-                ("fixed", 1, urwid.Filler(size_widget)),
-                ("fixed", 1, urwid.Filler(urwid.Edit("focus test edit: "))),
+                (urwid.WEIGHT, 70, term),
+                (1, urwid.Filler(size_widget)),
+                (1, urwid.Filler(urwid.Edit("focus test edit: "))),
             ]
         ),
     )
 
-    def set_title(widget, title):
+    def set_title(widget, title: str) -> None:
         mainframe.set_title(title)
 
-    def execute_quit(*args, **kwargs):
+    def execute_quit(*args, **kwargs) -> typing.NoReturn:
         raise urwid.ExitMainLoop()
 
-    def handle_key(key):
-        if key in ("q", "Q"):
+    def handle_key(key: str | tuple[str, int, int, int]) -> None:
+        if key in {"q", "Q"}:
             execute_quit()
 
-    def handle_resize(widget, size):
+    def handle_resize(widget, size: tuple[int, int]) -> None:
         size_widget.set_text(f"Terminal size: [{size[0]}, {size[1]}]")
 
     urwid.connect_signal(term, "title", set_title)
