@@ -40,7 +40,9 @@ from urwid.util import (
 )
 
 if typing.TYPE_CHECKING:
-    from collections.abc import Sequence
+    from collections.abc import Hashable, Iterable, Sequence
+
+    from typing_extensions import Literal
 
     from .widget import Widget
 
@@ -74,7 +76,7 @@ class CanvasCache:
     _refs: typing.ClassVar[
         dict[
             weakref.ReferenceType,
-            tuple[type[Widget], tuple[int, int] | tuple[int] | tuple[()], bool],
+            tuple[Widget, type[Widget], tuple[int, int] | tuple[int] | tuple[()], bool],
         ]
     ] = {}
     _deps: typing.ClassVar[dict[Widget, list[Widget]]] = {}
@@ -211,25 +213,12 @@ class Canvas:
     _finalized_error = CanvasError(
         "This canvas has been finalized. Use CompositeCanvas to wrap this canvas if you need to make changes."
     )
-    _renamed_error = CanvasError(
-        "The old Canvas class is now called TextCanvas. Canvas is now the base class for all canvas classes."
-    )
 
-    def __init__(
-        self,
-        value1: typing.Any = None,
-        value2: typing.Any = None,
-        value3: typing.Any = None,
-    ) -> None:
-        """
-        value1, value2, value3 -- if not None, raise a helpful error:
-            the old Canvas class is now called TextCanvas.
-        """
-        if value1 is not None:
-            raise self._renamed_error
+    def __init__(self) -> None:
+        """Base Canvas class"""
         self._widget_info = None
-        self.coords = {}
-        self.shortcuts = {}
+        self.coords: dict[str, tuple[int, int, tuple[Widget, int, int]]] = {}
+        self.shortcuts: dict[str, str] = {}
 
     def finalize(
         self,
@@ -268,11 +257,16 @@ class Canvas:
     @property
     def text(self) -> list[bytes]:
         """
-        Return the text content of the canvas as a list of strings,
-        one for each row.
+        Return the text content of the canvas as a list of strings, one for each row.
         """
         return [b"".join([text for (attr, cs, text) in row]) for row in self.content()]
 
+    @property
+    def decoded_text(self) -> Sequence[str]:
+        """Decoded text content of the canvas as a sequence of strings, one for each row."""
+        encoding = get_encoding()
+        return tuple(line.decode(encoding) for line in self.text)
+
     def _text_content(self):
         warnings.warn(
             f"Method `{self.__class__.__name__}._text_content` is deprecated, "
@@ -289,7 +283,7 @@ class Canvas:
         cols: int | None = None,
         rows: int | None = None,
         attr=None,
-    ):
+    ) -> Iterable[list[tuple[object, Literal["0", "U"] | None, bytes]]]:
         raise NotImplementedError()
 
     def cols(self):
@@ -298,7 +292,7 @@ class Canvas:
     def rows(self):
         raise NotImplementedError()
 
-    def content_delta(self):
+    def content_delta(self, other: Canvas):
         raise NotImplementedError()
 
     def get_cursor(self) -> tuple[int, int] | None:
@@ -307,7 +301,7 @@ class Canvas:
             return None
         return c[:2]  # trim off data part
 
-    def set_cursor(self, c):
+    def set_cursor(self, c: tuple[int, int] | None) -> None:
         if self.widget_info and self.cacheable:
             raise self._finalized_error
         if c is None:
@@ -318,13 +312,13 @@ class Canvas:
 
     cursor = property(get_cursor, set_cursor)
 
-    def get_pop_up(self):
+    def get_pop_up(self) -> tuple[int, int, tuple[Widget, int, int]] | None:
         c = self.coords.get("pop up", None)
         if not c:
             return None
         return c
 
-    def set_pop_up(self, w: Widget, left: int, top: int, overlay_width: int, overlay_height: int):
+    def set_pop_up(self, w: Widget, left: int, top: int, overlay_width: int, overlay_height: int) -> None:
         """
         This method adds pop-up information to the canvas.  This information
         is intercepted by a PopUpTarget widget higher in the chain to
@@ -347,7 +341,7 @@ class Canvas:
 
         self.coords["pop up"] = (left, top, (w, overlay_width, overlay_height))
 
-    def translate_coords(self, dx: int, dy: int):
+    def translate_coords(self, dx: int, dy: int) -> dict[str, tuple[int, int, tuple[Widget, int, int]]]:
         """
         Return coords shifted by (dx, dy).
         """
@@ -356,6 +350,25 @@ class Canvas:
             d[name] = (x + dx, y + dy, data)
         return d
 
+    def __repr__(self) -> str:
+        extra = [""]
+        with contextlib.suppress(BaseException):
+            extra.append(f"cols={self.cols()}")
+
+        with contextlib.suppress(BaseException):
+            extra.append(f"rows={self.rows()}")
+
+        if self.cursor:
+            extra.append(f"cursor={self.cursor}")
+
+        return f"<{self.__class__.__name__} finalized={bool(self.widget_info)}{' '.join(extra)} at 0x{id(self):X}>"
+
+    def __str__(self) -> str:
+        with contextlib.suppress(BaseException):
+            return "\n".join(self.decoded_text)
+
+        return repr(self)
+
 
 class TextCanvas(Canvas):
     """
@@ -365,8 +378,8 @@ class TextCanvas(Canvas):
     def __init__(
         self,
         text: Sequence[bytes] | None = None,
-        attr=None,
-        cs=None,
+        attr: list[list[tuple[Hashable | None, int]]] | None = None,
+        cs: list[list[tuple[Literal["0", "U"] | None, int]]] | None = None,
         cursor: tuple[int, int] | None = None,
         maxcol: int | None = None,
         check_width: bool = True,
@@ -387,7 +400,10 @@ class TextCanvas(Canvas):
             widths = []
             for t in text:
                 if not isinstance(t, bytes):
-                    raise CanvasError("Canvas text must be plain strings encoded in the screen's encoding", repr(text))
+                    raise CanvasError(
+                        "Canvas text must be plain strings encoded in the screen's encoding",
+                        repr(text),
+                    )
                 widths.append(calc_width(t, 0, len(t)))
         else:
             if not isinstance(maxcol, int):
@@ -462,10 +478,10 @@ class TextCanvas(Canvas):
         self,
         trim_left: int = 0,
         trim_top: int = 0,
-        cols: int = 0,
-        rows: int = 0,
-        attr_map=None,
-    ):
+        cols: int | None = 0,
+        rows: int | None = 0,
+        attr=None,
+    ) -> Iterable[tuple[object, Literal["0", "U"] | None, bytes]]:
         """
         Return the canvas content as a list of rows where each row
         is a list of (attr, cs, text) tuples.
@@ -507,13 +523,13 @@ class TextCanvas(Canvas):
             i = 0
             row = []
             for (a, cs), run in attr_cs:
-                if attr_map and a in attr_map:
-                    a = attr_map[a]  # noqa: PLW2901
+                if attr and a in attr:
+                    a = attr[a]  # noqa: PLW2901
                 row.append((a, cs, text[i : i + run]))
                 i += run
             yield row
 
-    def content_delta(self, other):
+    def content_delta(self, other: Canvas):
         """
         Return the differences between other and this canvas.
 
@@ -533,16 +549,16 @@ class BlankCanvas(Canvas):
     """
 
     def __init__(self) -> None:
-        super().__init__(None)
+        super().__init__()
 
     def content(
         self,
         trim_left: int = 0,
         trim_top: int = 0,
-        cols: int = 0,
-        rows: int = 0,
+        cols: int | None = 0,
+        rows: int | None = 0,
         attr=None,
-    ):
+    ) -> Iterable[list[tuple[object, Literal["0", "U"] | None, bytes]]]:
         """
         return (cols, rows) of spaces with default attributes.
         """
@@ -559,7 +575,7 @@ class BlankCanvas(Canvas):
     def rows(self) -> typing.NoReturn:
         raise NotImplementedError("BlankCanvas doesn't know its own size!")
 
-    def content_delta(self) -> typing.NoReturn:
+    def content_delta(self, other: Canvas) -> typing.NoReturn:
         raise NotImplementedError("BlankCanvas doesn't know its own size!")
 
 
@@ -594,7 +610,7 @@ class SolidCanvas(Canvas):
         cols: int | None = None,
         rows: int | None = None,
         attr=None,
-    ):
+    ) -> Iterable[list[tuple[object, Literal["0", "U"] | None, bytes]]]:
         if cols is None:
             cols = self.size[0]
         if rows is None:
@@ -641,8 +657,13 @@ class CompositeCanvas(Canvas):
         super().__init__()
 
         if canv is None:
-            self.shards = []
-            self.children = []
+            self.shards: list[
+                tuple[
+                    int,
+                    list[tuple[int, int, int, int, dict[Hashable | None, Hashable] | None, Canvas]],
+                ]
+            ] = []
+            self.children: list[tuple[int, int, Canvas, typing.Any]] = []
         else:
             if hasattr(canv, "shards"):
                 self.shards = canv.shards
@@ -653,6 +674,21 @@ class CompositeCanvas(Canvas):
             for shortcut in canv.shortcuts:
                 self.shortcuts[shortcut] = "wrap"
 
+    def __repr__(self) -> str:
+        extra = [""]
+        with contextlib.suppress(BaseException):
+            extra.append(f"cols={self.cols()}")
+
+        with contextlib.suppress(BaseException):
+            extra.append(f"rows={self.rows()}")
+
+        if self.cursor:
+            extra.append(f"cursor={self.cursor}")
+        if self.children:
+            extra.append(f"children=({', '.join(repr(canv) for _, _, canv, _ in self.children)})")
+
+        return f"<{self.__class__.__name__} finalized={bool(self.widget_info)}{' '.join(extra)} at 0x{id(self):X}>"
+
     def rows(self) -> int:
         for r, cv in self.shards:
             if not isinstance(r, int):
@@ -670,7 +706,14 @@ class CompositeCanvas(Canvas):
             raise TypeError(cols)
         return cols
 
-    def content(self):
+    def content(
+        self,
+        trim_left: int = 0,
+        trim_top: int = 0,
+        cols: int | None = None,
+        rows: int | None = None,
+        attr=None,
+    ) -> Iterable[list[tuple[object, Literal["0", "U"] | None, bytes]]]:
         """
         Return the canvas content as a list of rows where each row
         is a list of (attr, cs, text) tuples.
@@ -687,7 +730,7 @@ class CompositeCanvas(Canvas):
             # prepare next shard tail
             shard_tail = shard_body_tail(num_rows, sbody)
 
-    def content_delta(self, other):
+    def content_delta(self, other: Canvas):
         """
         Return the differences between other and this canvas.
         """
@@ -803,7 +846,7 @@ class CompositeCanvas(Canvas):
                 self.shards = self.shards[:]
             self.shards.append((bottom, [(0, 0, cols, bottom, None, blank_canvas)]))
 
-    def overlay(self, other, left: int, top: int) -> None:
+    def overlay(self, other: CompositeCanvas, left: int, top: int) -> None:
         """Overlay other onto this canvas."""
         if self.widget_info:
             raise self._finalized_error
@@ -847,14 +890,14 @@ class CompositeCanvas(Canvas):
 
         self.coords.update(other.translate_coords(left, top))
 
-    def fill_attr(self, a) -> None:
+    def fill_attr(self, a: Hashable) -> None:
+        """
+        Apply attribute a to all areas of this canvas with default attribute currently set to None,
+        leaving other attributes intact.
         """
-        Apply attribute a to all areas of this canvas with default
-        attribute currently set to None, leaving other attributes
-        intact."""
         self.fill_attr_apply({None: a})
 
-    def fill_attr_apply(self, mapping) -> None:
+    def fill_attr_apply(self, mapping: dict[Hashable | None, Hashable]) -> None:
         """
         Apply an attribute-mapping dictionary to the canvas.
 
@@ -915,7 +958,7 @@ def shard_body_tail(num_rows: int, sbody):
     """
     shard_tail = []
     col_gap = 0
-    done_rows = 0
+
     for done_rows, content_iter, cview in sbody:
         cols, rows = cview[2:4]
         done_rows += num_rows  # noqa: PLW2901
@@ -1167,7 +1210,7 @@ def cview_trim_cols(cv, cols: int):
     return cv[:2] + (cols,) + cv[3:]
 
 
-def CanvasCombine(canvas_info):
+def CanvasCombine(canvas_info: Iterable[tuple[Canvas, typing.Any, bool]]) -> CompositeCanvas:
     """Stack canvases in l vertically and return resulting canvas.
 
     :param canvas_info: list of (canvas, position, focus) tuples:
@@ -1184,8 +1227,8 @@ def CanvasCombine(canvas_info):
     children = []
     row = 0
     focus_index = 0
-    n = 0
-    for canv, pos, focus in clist:
+
+    for n, (canv, pos, focus) in enumerate(clist):
         if focus:
             focus_index = n
         children.append((0, row, canv, pos))
@@ -1194,7 +1237,6 @@ def CanvasCombine(canvas_info):
         for shortcut in canv.shortcuts:
             combined_canvas.shortcuts[shortcut] = pos
         row += canv.rows()
-        n += 1
 
     if focus_index:
         children = [children[focus_index]] + children[:focus_index] + children[focus_index + 1 :]
@@ -1204,7 +1246,7 @@ def CanvasCombine(canvas_info):
     return combined_canvas
 
 
-def CanvasOverlay(top_c, bottom_c, left: int, top: int):
+def CanvasOverlay(top_c: Canvas, bottom_c: Canvas, left: int, top: int) -> CompositeCanvas:
     """
     Overlay canvas top_c onto bottom_c at position (left, top).
     """
@@ -1217,7 +1259,7 @@ def CanvasOverlay(top_c, bottom_c, left: int, top: int):
     return overlayed_canvas
 
 
-def CanvasJoin(canvas_info):
+def CanvasJoin(canvas_info: Iterable[tuple[Canvas, typing.Any, bool, int]]) -> CompositeCanvas:
     """
     Join canvases in l horizontally. Return result.
 
@@ -1236,8 +1278,8 @@ def CanvasJoin(canvas_info):
     l2 = []
     focus_item = 0
     maxrow = 0
-    n = 0
-    for canv, pos, focus, cols in canvas_info:
+
+    for n, (canv, pos, focus, cols) in enumerate(canvas_info):
         rows = canv.rows()
         pad_right = cols - canv.cols()
         if focus:
@@ -1245,7 +1287,6 @@ def CanvasJoin(canvas_info):
         if rows > maxrow:
             maxrow = rows
         l2.append((canv, pos, pad_right, rows))
-        n += 1
 
     shard_lists = []
     children = []
@@ -1272,7 +1313,12 @@ def CanvasJoin(canvas_info):
     return joined_canvas
 
 
-def apply_text_layout(text, attr, ls, maxcol: int):
+def apply_text_layout(
+    text: str | bytes,
+    attr: list[tuple[Hashable, int]],
+    ls: list[list[tuple[int, int, int | bytes] | tuple[int, int | None]]],
+    maxcol: int,
+) -> TextCanvas:
     t = []
     a = []
     c = []
@@ -1284,7 +1330,7 @@ def apply_text_layout(text, attr, ls, maxcol: int):
     aw.k = 0  # counter for moving through elements of a
     aw.off = 0  # current offset into text of attr[ak]
 
-    def arange(start_offs: int, end_offs: int):
+    def arange(start_offs: int, end_offs: int) -> list[tuple[Hashable | None, int]]:
         """Return an attribute list for the range of text specified."""
         if start_offs < aw.off:
             aw.k = 0
