@@ -1,5 +1,3 @@
-#!/usr/bin/python
-#
 # Urwid MonitoredList class
 #    Copyright (C) 2004-2011  Ian Ward
 #
@@ -17,27 +15,47 @@
 #    License along with this library; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# Urwid web site: http://excess.org/urwid/
+# Urwid web site: https://urwid.org/
 
-from urwid.compat import PYTHON3
+from __future__ import annotations
+
+import functools
+import typing
+import warnings
+
+if typing.TYPE_CHECKING:
+    from collections.abc import Callable, Collection, Iterator
+
+    from typing_extensions import Concatenate, ParamSpec
+
+    ArgSpec = ParamSpec("ArgSpec")
+    Ret = typing.TypeVar("Ret")
+
+_T = typing.TypeVar("_T")
 
 
-def _call_modified(fn):
-    def call_modified_wrapper(self, *args, **kwargs):
+def _call_modified(
+    fn: Callable[Concatenate[MonitoredList, ArgSpec], Ret]
+) -> Callable[Concatenate[MonitoredList, ArgSpec], Ret]:
+    @functools.wraps(fn)
+    def call_modified_wrapper(self: MonitoredList, *args: ArgSpec.args, **kwargs: ArgSpec.kwargs) -> Ret:
         rval = fn(self, *args, **kwargs)
         self._modified()
         return rval
+
     return call_modified_wrapper
 
-class MonitoredList(list):
+
+class MonitoredList(typing.List[_T], typing.Generic[_T]):
     """
     This class can trigger a callback any time its contents are changed
     with the usual list operations append, extend, etc.
     """
-    def _modified(self):
+
+    def _modified(self) -> None:
         pass
 
-    def set_modified_callback(self, callback):
+    def set_modified_callback(self, callback: Callable[[], typing.Any]) -> None:
         """
         Assign a callback function with no parameters that is called any
         time the list is modified.  Callback's return value is ignored.
@@ -58,38 +76,42 @@ class MonitoredList(list):
         >>> ml
         MonitoredList([1, 2, 12, 13])
         """
-        self._modified = callback
+        self._modified = callback  # monkeypatch
 
-    def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, list(self))
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}({list(self)!r})"
 
-    __add__ = _call_modified(list.__add__)
-    __delitem__ = _call_modified(list.__delitem__)
-    if not PYTHON3:
-        __delslice__ = _call_modified(list.__delslice__)
-    __iadd__ = _call_modified(list.__iadd__)
-    __imul__ = _call_modified(list.__imul__)
-    __rmul__ = _call_modified(list.__rmul__)
-    __setitem__ = _call_modified(list.__setitem__)
-    if not PYTHON3:
-        __setslice__ = _call_modified(list.__setslice__)
-    append = _call_modified(list.append)
-    extend = _call_modified(list.extend)
-    insert = _call_modified(list.insert)
-    pop = _call_modified(list.pop)
-    remove = _call_modified(list.remove)
-    reverse = _call_modified(list.reverse)
-    sort = _call_modified(list.sort)
-    if hasattr(list, 'clear'):
-        clear = _call_modified(list.clear)
+    # noinspection PyMethodParameters
+    def __rich_repr__(inner_self) -> Iterator[tuple[str | None, typing.Any] | typing.Any]:
+        for idx in range(len(inner_self)):
+            yield None, inner_self[idx]
+
+    __add__ = _call_modified(list.__add__)  # type: ignore[assignment]  # magic like old __super__
+    __delitem__ = _call_modified(list.__delitem__)  # type: ignore[assignment]  # magic like old __super__
+
+    __iadd__ = _call_modified(list.__iadd__)  # type: ignore[assignment]  # magic like old __super__
+    __imul__ = _call_modified(list.__imul__)  # type: ignore[assignment]  # magic like old __super__
+    __rmul__ = _call_modified(list.__rmul__)  # type: ignore[assignment]  # magic like old __super__
+    __setitem__ = _call_modified(list.__setitem__)  # type: ignore[assignment]  # magic like old __super__
+
+    append = _call_modified(list.append)  # type: ignore[assignment]  # magic like old __super__
+    extend = _call_modified(list.extend)  # type: ignore[assignment]  # magic like old __super__
+    insert = _call_modified(list.insert)  # type: ignore[assignment]  # magic like old __super__
+    pop = _call_modified(list.pop)  # type: ignore[assignment]  # magic like old __super__
+    remove = _call_modified(list.remove)  # type: ignore[assignment]  # magic like old __super__
+    reverse = _call_modified(list.reverse)  # type: ignore[assignment]  # magic like old __super__
+    sort = _call_modified(list.sort)  # type: ignore[assignment]  # magic like old __super__
+    if hasattr(list, "clear"):
+        clear = _call_modified(list.clear)  # type: ignore[assignment]  # magic like old __super__
 
 
-class MonitoredFocusList(MonitoredList):
+class MonitoredFocusList(MonitoredList[_T], typing.Generic[_T]):
     """
     This class can trigger a callback any time its contents are modified,
     before and/or after modification, and any time the focus index is changed.
     """
-    def __init__(self, *argl, **argd):
+
+    def __init__(self, *args, focus: int = 0, **kwargs) -> None:
         """
         This is a list that tracks one item as the focus item.  If items
         are inserted or removed it will update the focus.
@@ -110,31 +132,35 @@ class MonitoredFocusList(MonitoredList):
         >>> ml
         MonitoredFocusList([], focus=None)
         """
-        focus = argd.pop('focus', 0)
 
-        super(MonitoredFocusList, self).__init__(*argl, **argd)
+        super().__init__(*args, **kwargs)
 
         self._focus = focus
         self._focus_modified = lambda ml, indices, new_items: None
 
-    def __repr__(self):
-        return "%s(%r, focus=%r)" % (
-            self.__class__.__name__, list(self), self.focus)
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}({list(self)!r}, focus={self.focus!r})"
 
-    def _get_focus(self):
+    @property
+    def focus(self) -> int | None:
         """
+        Get/set the focus index.  This value is read as None when the list
+        is empty, and may only be set to a value between 0 and len(self)-1
+        or an IndexError will be raised.
+
         Return the index of the item "in focus" or None if
         the list is empty.
 
-        >>> MonitoredFocusList([1,2,3], focus=2)._get_focus()
+        >>> MonitoredFocusList([1,2,3], focus=2).focus
         2
-        >>> MonitoredFocusList()._get_focus()
+        >>> MonitoredFocusList().focus
         """
         if not self:
             return None
         return self._focus
 
-    def _set_focus(self, index):
+    @focus.setter
+    def focus(self, index: int) -> None:
         """
         index -- index into this list, any index out of range will
             raise an IndexError, except when the list is empty and
@@ -147,11 +173,11 @@ class MonitoredFocusList(MonitoredList):
         instance with set_focus_changed_callback().
 
         >>> ml = MonitoredFocusList([9, 10, 11])
-        >>> ml._set_focus(2); ml._get_focus()
+        >>> ml.focus = 2; ml.focus
         2
-        >>> ml._set_focus(0); ml._get_focus()
+        >>> ml.focus = 0; ml.focus
         0
-        >>> ml._set_focus(-2)
+        >>> ml.focus = -2
         Traceback (most recent call last):
         ...
         IndexError: focus index is out of range: -2
@@ -160,24 +186,36 @@ class MonitoredFocusList(MonitoredList):
             self._focus = 0
             return
         if index < 0 or index >= len(self):
-            raise IndexError('focus index is out of range: %s' % (index,))
+            raise IndexError(f"focus index is out of range: {index}")
         if index != int(index):
-            raise IndexError('invalid focus index: %s' % (index,))
+            raise IndexError(f"invalid focus index: {index}")
         index = int(index)
         if index != self._focus:
             self._focus_changed(index)
         self._focus = index
 
-    focus = property(_get_focus, _set_focus, doc="""
-        Get/set the focus index.  This value is read as None when the list
-        is empty, and may only be set to a value between 0 and len(self)-1
-        or an IndexError will be raised.
-        """)
+    def _get_focus(self) -> int | None:
+        warnings.warn(
+            f"method `{self.__class__.__name__}._get_focus` is deprecated, "
+            f"please use `{self.__class__.__name__}.focus` property",
+            DeprecationWarning,
+            stacklevel=3,
+        )
+        return self.focus
 
-    def _focus_changed(self, new_focus):
+    def _set_focus(self, index: int) -> None:
+        warnings.warn(
+            f"method `{self.__class__.__name__}._set_focus` is deprecated, "
+            f"please use `{self.__class__.__name__}.focus` property",
+            DeprecationWarning,
+            stacklevel=3,
+        )
+        self.focus = index
+
+    def _focus_changed(self, new_focus: int) -> None:
         pass
 
-    def set_focus_changed_callback(self, callback):
+    def set_focus_changed_callback(self, callback: Callable[[int], typing.Any]) -> None:
         """
         Assign a callback to be called when the focus index changes
         for any reason.  The callback is in the form:
@@ -204,12 +242,12 @@ class MonitoredFocusList(MonitoredList):
         >>> ml
         MonitoredFocusList([12, 13, 14, 2, 3, 10], focus=5)
         """
-        self._focus_changed = callback
+        self._focus_changed = callback  # Monkeypatch
 
-    def _validate_contents_modified(self, indices, new_items):
+    def _validate_contents_modified(self, indices: tuple[int, int, int], new_items: Collection[_T]) -> int | None:
         return None
 
-    def set_validate_contents_modified(self, callback):
+    def set_validate_contents_modified(self, callback: Callable[[tuple[int, int, int], Collection[_T]], int | None]):
         """
         Assign a callback function to handle validating changes to the list.
         This may raise an exception if the change should not be performed.
@@ -225,9 +263,9 @@ class MonitoredFocusList(MonitoredList):
             empty if items are being removed, if step==1 this list may
             contain any number of items
         """
-        self._validate_contents_modified = callback
+        self._validate_contents_modified = callback  # Monkeypatch
 
-    def _adjust_focus_on_contents_modified(self, slc, new_items=()):
+    def _adjust_focus_on_contents_modified(self, slc: slice, new_items: Collection[_T] = ()) -> int:
         """
         Default behaviour is to move the focus to the item following
         any removed items, unless that item was simply replaced.
@@ -238,7 +276,7 @@ class MonitoredFocusList(MonitoredList):
         """
         num_new_items = len(new_items)
         start, stop, step = indices = slc.indices(len(self))
-        num_removed = len(range(*indices))
+        num_removed = len(list(range(*indices)))
 
         focus = self._validate_contents_modified(indices, new_items)
         if focus is not None:
@@ -252,20 +290,20 @@ class MonitoredFocusList(MonitoredList):
             if stop <= focus:
                 focus += num_new_items - (stop - start)
 
-        else:
+        else:  # noqa: PLR5501
             if not num_new_items:
                 # extended slice being removed
                 if focus in range(start, stop, step):
                     focus += 1
 
                 # adjust for removed items
-                focus -= len(range(start, min(focus, stop), step))
+                focus -= len(list(range(start, min(focus, stop), step)))
 
-        return min(focus, len(self) + num_new_items - num_removed -1)
+        return min(focus, len(self) + num_new_items - num_removed - 1)
 
     # override all the list methods that modify the list
 
-    def __delitem__(self, y):
+    def __delitem__(self, y: int | slice) -> None:
         """
         >>> ml = MonitoredFocusList([0,1,2,3,4], focus=2)
         >>> del ml[3]; ml
@@ -294,16 +332,20 @@ class MonitoredFocusList(MonitoredList):
         if isinstance(y, slice):
             focus = self._adjust_focus_on_contents_modified(y)
         else:
-            focus = self._adjust_focus_on_contents_modified(slice(y,
-                y+1 or None))
-        rval = super(MonitoredFocusList, self).__delitem__(y)
-        self._set_focus(focus)
-        return rval
+            focus = self._adjust_focus_on_contents_modified(slice(y, y + 1 or None))
+        super().__delitem__(y)
+        self.focus = focus
 
-    def __setitem__(self, i, y):
+    @typing.overload
+    def __setitem__(self, i: int, y: _T) -> None: ...
+
+    @typing.overload
+    def __setitem__(self, i: slice, y: Collection[_T]) -> None: ...
+
+    def __setitem__(self, i: int | slice, y: _T | Collection[_T]) -> None:
         """
         >>> def modified(indices, new_items):
-        ...     print "range%r <- %r" % (indices, new_items)
+        ...     print(f"range{indices!r} <- {new_items!r}" )
         >>> ml = MonitoredFocusList([0,1,2,3], focus=2)
         >>> ml.set_validate_contents_modified(modified)
         >>> ml[0] = 9
@@ -332,22 +374,14 @@ class MonitoredFocusList(MonitoredList):
         if isinstance(i, slice):
             focus = self._adjust_focus_on_contents_modified(i, y)
         else:
-            focus = self._adjust_focus_on_contents_modified(slice(i, i+1 or None), [y])
-        rval = super(MonitoredFocusList, self).__setitem__(i, y)
-        self._set_focus(focus)
-        return rval
+            focus = self._adjust_focus_on_contents_modified(slice(i, i + 1 or None), [y])
+        super().__setitem__(i, y)
+        self.focus = focus
 
-    if not PYTHON3:
-        def __delslice__(self, i, j):
-            return self.__delitem__(slice(i,j))
-
-        def __setslice__(self, i, j, y):
-            return self.__setitem__(slice(i, j), y)
-
-    def __imul__(self, n):
+    def __imul__(self, n: int):
         """
         >>> def modified(indices, new_items):
-        ...     print "range%r <- %r" % (indices, list(new_items))
+        ...     print(f"range{indices!r} <- {list(new_items)!r}" )
         >>> ml = MonitoredFocusList([0,1,2], focus=2)
         >>> ml.set_validate_contents_modified(modified)
         >>> ml *= 3
@@ -356,49 +390,44 @@ class MonitoredFocusList(MonitoredList):
         MonitoredFocusList([0, 1, 2, 0, 1, 2, 0, 1, 2], focus=2)
         >>> ml *= 0
         range(0, 9, 1) <- []
-        >>> print ml.focus
+        >>> print(ml.focus)
         None
         """
         if n > 0:
-            focus = self._adjust_focus_on_contents_modified(
-                slice(len(self), len(self)), list(self)*(n-1))
-        else: # all contents are being removed
+            focus = self._adjust_focus_on_contents_modified(slice(len(self), len(self)), list(self) * (n - 1))
+        else:  # all contents are being removed
             focus = self._adjust_focus_on_contents_modified(slice(0, len(self)))
-        rval = super(MonitoredFocusList, self).__imul__(n)
-        self._set_focus(focus)
+        rval = super().__imul__(n)
+        self.focus = focus
         return rval
 
-    def append(self, item):
+    def append(self, item: _T) -> None:
         """
         >>> def modified(indices, new_items):
-        ...     print "range%r <- %r" % (indices, new_items)
+        ...     print(f"range{indices!r} <- {new_items!r}" )
         >>> ml = MonitoredFocusList([0,1,2], focus=2)
         >>> ml.set_validate_contents_modified(modified)
         >>> ml.append(6)
         range(3, 3, 1) <- [6]
         """
-        focus = self._adjust_focus_on_contents_modified(
-            slice(len(self), len(self)), [item])
-        rval = super(MonitoredFocusList, self).append(item)
-        self._set_focus(focus)
-        return rval
+        focus = self._adjust_focus_on_contents_modified(slice(len(self), len(self)), [item])
+        super().append(item)
+        self.focus = focus
 
-    def extend(self, items):
+    def extend(self, items: Collection[_T]) -> None:
         """
         >>> def modified(indices, new_items):
-        ...     print "range%r <- %r" % (indices, list(new_items))
+        ...     print(f"range{indices!r} <- {list(new_items)!r}" )
         >>> ml = MonitoredFocusList([0,1,2], focus=2)
         >>> ml.set_validate_contents_modified(modified)
         >>> ml.extend((6,7,8))
         range(3, 3, 1) <- [6, 7, 8]
         """
-        focus = self._adjust_focus_on_contents_modified(
-            slice(len(self), len(self)), items)
-        rval = super(MonitoredFocusList, self).extend(items)
-        self._set_focus(focus)
-        return rval
+        focus = self._adjust_focus_on_contents_modified(slice(len(self), len(self)), items)
+        super().extend(items)
+        self.focus = focus
 
-    def insert(self, index, item):
+    def insert(self, index: int, item: _T) -> None:
         """
         >>> ml = MonitoredFocusList([0,1,2,3], focus=2)
         >>> ml.insert(-1, -1); ml
@@ -408,13 +437,11 @@ class MonitoredFocusList(MonitoredList):
         >>> ml.insert(3, -3); ml
         MonitoredFocusList([-2, 0, 1, -3, 2, -1, 3], focus=4)
         """
-        focus = self._adjust_focus_on_contents_modified(slice(index, index),
-            [item])
-        rval = super(MonitoredFocusList, self).insert(index, item)
-        self._set_focus(focus)
-        return rval
+        focus = self._adjust_focus_on_contents_modified(slice(index, index), [item])
+        super().insert(index, item)
+        self.focus = focus
 
-    def pop(self, index=-1):
+    def pop(self, index: int = -1) -> _T:
         """
         >>> ml = MonitoredFocusList([-2,0,1,-3,2,3], focus=4)
         >>> ml.pop(3); ml
@@ -430,13 +457,12 @@ class MonitoredFocusList(MonitoredList):
         2
         MonitoredFocusList([0, 1], focus=1)
         """
-        focus = self._adjust_focus_on_contents_modified(slice(index,
-            index+1 or None))
-        rval = super(MonitoredFocusList, self).pop(index)
-        self._set_focus(focus)
+        focus = self._adjust_focus_on_contents_modified(slice(index, index + 1 or None))
+        rval = super().pop(index)
+        self.focus = focus
         return rval
 
-    def remove(self, value):
+    def remove(self, value: _T) -> None:
         """
         >>> ml = MonitoredFocusList([-2,0,1,-3,2,-1,3], focus=4)
         >>> ml.remove(-3); ml
@@ -447,50 +473,46 @@ class MonitoredFocusList(MonitoredList):
         MonitoredFocusList([0, 1, 2, -1], focus=2)
         """
         index = self.index(value)
-        focus = self._adjust_focus_on_contents_modified(slice(index,
-            index+1 or None))
-        rval = super(MonitoredFocusList, self).remove(value)
-        self._set_focus(focus)
-        return rval
+        focus = self._adjust_focus_on_contents_modified(slice(index, index + 1 or None))
+        super().remove(value)
+        self.focus = focus
 
-    def reverse(self):
+    def reverse(self) -> None:
         """
         >>> ml = MonitoredFocusList([0,1,2,3,4], focus=1)
         >>> ml.reverse(); ml
         MonitoredFocusList([4, 3, 2, 1, 0], focus=3)
         """
-        rval = super(MonitoredFocusList, self).reverse()
-        self._set_focus(max(0, len(self) - self._focus - 1))
+        rval = super().reverse()
+        self.focus = max(0, len(self) - self._focus - 1)
         return rval
 
-    def sort(self, **kwargs):
+    def sort(self, **kwargs) -> None:
         """
         >>> ml = MonitoredFocusList([-2,0,1,-3,2,-1,3], focus=4)
         >>> ml.sort(); ml
         MonitoredFocusList([-3, -2, -1, 0, 1, 2, 3], focus=5)
         """
         if not self:
-            return
+            return None
         value = self[self._focus]
-        rval = super(MonitoredFocusList, self).sort(**kwargs)
-        self._set_focus(self.index(value))
+        rval = super().sort(**kwargs)
+        self.focus = self.index(value)
         return rval
 
-    if hasattr(list, 'clear'):
-        def clear(self):
+    if hasattr(list, "clear"):
+
+        def clear(self) -> None:
             focus = self._adjust_focus_on_contents_modified(slice(0, 0))
-            rval = super(MonitoredFocusList, self).clear()
-            self._set_focus(focus)
-            return rval
-
-
-
+            super().clear()
+            self.focus = focus
 
 
 def _test():
     import doctest
+
     doctest.testmod()
 
-if __name__=='__main__':
-    _test()
 
+if __name__ == "__main__":
+    _test()
